#include "CuTest.h"
#include "aos_util.h"
#include "aos_http_io.h"
#include "oss_test_util.h"

static const char *TEST_FILE = ".\\oss_c_sdk_test";

void test_open_file_read(CuTest *tc)
{
    int code;
    size_t elen;
    char *buf;
    aos_pool_t *p;
    aos_file_buf_t *fb;
    aos_http_request_t *req;

    aos_pool_create(&p, NULL);

    fb = aos_create_file_buf(p);

    // not exist
    code = aos_open_file_for_read(p, "ccccc.txt", fb);
    
    code = aos_open_file_for_read(p, TEST_FILE, fb);
    CuAssertIntEquals(tc, AOSE_OK, code);
    
    CuAssertIntEquals(tc, fb->file_pos, 0);
    CuAssertTrue(tc, fb->owner);
    CuAssertTrue(tc, fb->file_last > 100);
    
    req = aos_http_request_create(p);
    req->file_buf = fb;
    buf = (char *)aos_pcalloc(p, fb->file_last);
    elen = aos_read_http_body_file(req, buf, fb->file_last);
    CuAssertIntEquals(tc, fb->file_last, elen);

    elen = aos_read_http_body_file(req, buf, fb->file_last);
    CuAssertIntEquals(tc, 0, elen);

    apr_file_close(fb->file);
    aos_pool_destroy(p);
}

void test_open_file_write(CuTest *tc)
{
    int elen;
    aos_pool_t *p;
    aos_buf_t *b;
    aos_http_response_t *resp;

    aos_pool_create(&p, NULL);

    resp = aos_http_response_create(p);
    resp->file_path = "./oss_c_sdk_test_file_write";

    b = make_random_buf(p, 1024);
    elen = aos_write_http_body_file(resp, (char *)b->pos, aos_buf_size(b));
    CuAssertIntEquals(tc, aos_buf_size(b), elen);
    CuAssertIntEquals(tc, aos_buf_size(b), resp->body_len);
    CuAssertIntEquals(tc, aos_buf_size(b), resp->file_buf->file_last);

    apr_file_close(resp->file_buf->file);
    aos_pool_destroy(p);
}

CuSuite *test_file()
{
    CuSuite* suite = CuSuiteNew();

    SUITE_ADD_TEST(suite, test_open_file_read);
    SUITE_ADD_TEST(suite, test_open_file_write);

    return suite;
}
